import pkg_resources

from ._client import Client

__author__ = "Anders Innovations"

try:
    __version__ = pkg_resources.get_distribution(__name__).version
except pkg_resources.DistributionNotFound:
    __version__ = None

__all__ = [
    'Client',
]



# -*- coding: utf-8 -*-


class ApiError(Exception):
    def __init__(self, message, code, response):
        super(ApiError, self).__init__(message)
        self.code = code
        self.response = response



from __future__ import unicode_literals

CUSTOMS_DECLARATION_SCHEMA = {
    "type": "object",
    "properties": {
        "sender": {
            "type": "object",
            "required": ["name", "city", "country"],
            "properties": {
                "quickId": {"type": "string"},
                "orgNo": {"type": "string"},
                "vatNo": {"type": "string"},
                "name": {"type": "string"},
                "address1": {"type": "string"},
                "address2": {"type": "string"},
                "zipcode": {"type": "string"},
                "city": {"type": "string"},
                "state": {"type": "string"},
                "country": {"type": "string"},
                "contact": {"type": "string"},
                "phone": {"type": "string"},
                "fax": {"type": "string"},
                "email": {"type": "string"},
                "mobile": {"type": "string"},
                "doorCode": {"type": "string"},
                "serviceType": {"type": "string"},
                "serviceCode": {"type": "string"},
                "openingHours": {"type": "string"}
            }
        },
        "receiver": {
            "type": "object",
            "required": ["name", "city", "country"],
            "properties": {
                "quickId": {"type": "string"},
                "orgNo": {"type": "string"},
                "vatNo": {"type": "string"},
                "name": {"type": "string"},
                "address1": {"type": "string"},
                "address2": {"type": "string"},
                "zipcode": {"type": "string"},
                "city": {"type": "string"},
                "state": {"type": "string"},
                "country": {"type": "string"},
                "contact": {"type": "string"},
                "phone": {"type": "string"},
                "fax": {"type": "string"},
                "email": {"type": "string"},
                "mobile": {"type": "string"},
                "doorCode": {"type": "string"},
                "serviceType": {"type": "string"},
                "serviceCode": {"type": "string"},
                "openingHours": {"type": "string"}
            }
        },
        "printSet": {
            "type": "array",
            "items": {
                "type": "string"
            }
        },
        "parcelCount": {"type": "integer", "minimum": 0},
        "reference": {"type": "string"},
        "sourceCountryCode": {"type": "string"},
        "destinationCountryCode": {"type": "string"},
        "shippingCodeBorder": {"type": "string"},
        "shippingCodeDomestic": {"type": "string"},
        "finance1": {"type": "string"},
        "finance2": {"type": "string"},
        "declarantCity": {"type": "string"},
        "declarantDate": {"type": "string"},
        "declarant": {"type": "string"},
        "jobTitle": {"type": "string"},
        "edocNormal": {"type": "boolean"},
        "representative1": {"type": "string"},
        "representative2": {"type": "string"},
        "representativeOrgNo": {"type": "string"},
        "invoiceType": {"type": "string"},
        "invoiceNo": {"type": "string"},
        "discount": {"type": "number", "minimum": 0},
        "freightCharges": {"type": "number", "minimum": 0},
        "insuranceCharges": {"type": "number", "minimum": 0},
        "otherCharges": {"type": "number", "minimum": 0},
        "exportLicenseNo": {"type": "string"},
        "exportDeclaration1": {"type": "string"},
        "exportDeclaration2": {"type": "string"},
        "generalNote1": {"type": "string"},
        "generalNote2": {"type": "string"},
        "generalNote3": {"type": "string"},
        "generalNote4": {"type": "string"},
        "currencyCode": {"type": "string"},
        "importExportType": {"type": "string"},
        "invoiceDeclaration1": {"type": "string"},
        "invoiceDeclaration2": {"type": "string"},
        "importLicenseNo": {"type": "string"},
        "certificate": {"type": "string"},
        "explanation": {"type": "string"},
        "termCode": {"type": "string"},
        "termLocation": {"type": "string"},
        "lines": {
            "type": "array",
            "items": {
                "type": "object",
                "required": ["copies"],
                "properties": {
                    "valuesPerItem": {"type": "boolean"},
                    "statNo": {"type": "string"},
                    "subStatNo1": {"type": "string"},
                    "subStatNo2": {"type": "string"},
                    "procedure": {"type": "string"},
                    "value": {"type": "number"},
                    "goodsMark1": {"type": "string"},
                    "goodsMark2": {"type": "string"},
                    "goodsMark3": {"type": "string"},
                    "goodsMark4": {"type": "string"},
                    "goodsMark5": {"type": "string"},
                    "goodsMark6": {"type": "string"},
                    "contents": {"type": "string"},
                    "copies": {"type": "integer", "minimum": 1},
                    "netWeight": {"type": "number", "minimum": 0},
                    "sourceCountryCode": {"type": "string"},
                    "otherUnit": {"type": "string"},
                    "otherQuantity": {"type": "number", "minimum": 0}
                }
            }
        }
    }
}

ADDRESS_SCHEMA = {
    "type": "object",
    "oneOf": [
        {"required": ["name", "city", "country"]},
        {"required": ["quickId"]}
    ],
    "properties": {
        "quickId": {"type": "string"},
        "orgNo": {"type": "string"},
        "vatNo": {"type": "string"},
        "name": {"type": "string"},
        "address1": {"type": "string"},
        "address2": {"type": "string"},
        "zipcode": {"type": "string"},
        "city": {"type": "string"},
        "state": {"type": "string"},
        "country": {"type": "string"},
        "contact": {"type": "string"},
        "phone": {"type": "string"},
        "fax": {"type": "string"},
        "email": {"type": "string"},
        "mobile": {"type": "string"},
        "doorCode": {"type": "string"},
        "serviceType": {"type": "string"},
        "serviceCode": {"type": "string"},
        "openingHours": {"type": "string"}
    },
}

AGENT_ITEM_SCHEMA = {
    "type": "object",
    "required": ["id", "name", "city", "countryCode"],
    "properties": {
        "id": {"type": "string"},
        "name": {"type": "string"},
        "address1": {"type": ["string", "null"]},
        "address2": {"type": ["string", "null"]},
        "zipcode": {"type": ["string", "null"]},
        "city": {"type": "string"},
        "state": {"type": ["string", "null"]},
        "countryCode": {"type": "string"},
        "contact": {"type": ["string", "null"]},
        "phone": {"type": ["string", "null"]},
        "fax": {"type": ["string", "null"]},
        "email": {"type": ["string", "null"]},
        "sms": {"type": ["string", "null"]},
        "serviceType": {"type": ["string", "null"]},
        "serviceCode": {"type": ["string", "null"]},
        "openingHours": {"type": ["string", "null"]},
    },
}

AGENTS_SCHEMA = {
    "type": "array",
    "$schema": "http://json-schema.org/draft-04/schema#",
    "items": AGENT_ITEM_SCHEMA,
}

PARTNER_SCHEMA = {
    "type": "array",
    "items": {
        "type": "object",
        "required": ["id"],
        "properties": {
            "id": {"type": "string"},
            "agentNo": {"type": "string"},
            "custNo": {"type": "string"},
            "custNoIssuerCode": {"type": "string"},
            "palletRegNo": {"type": "string"},
            "ediAddress": {"type": "string"},
            "senderId": {"type": "string"},
            "bookingId": {"type": "string"},
            "bookingOffice": {"type": "string"},
            "bookingEmail": {"type": "string"},
            "sourceCode": {"type": "string"},
            "ediUserId": {"type": "string"},
            "ediPassword": {"type": "string"},
            "ediKey": {"type": "string"},
            "externalIdentifier": {"type": "string"}
        }
    }
}

PARCELS_SCHEMA = {
    "type": "array",
    "items": {
        "type": "object",
        "required": ["copies"],
        "properties": {
            "valuePerParcel": {"type": "boolean"},
            "copies": {"type": "integer", "minimum": 1},
            "marking": {"type": "string"},
            "packageCode": {"type": "string"},
            "packageText": {"type": "string"},
            "weight": {"type": "number", "minimum": 0},
            "volume": {"type": "number", "minimum": 0},
            "length": {"type": "number", "minimum": 0},
            "width": {"type": "number", "minimum": 0},
            "height": {"type": "number", "minimum": 0},
            "loadingMeters": {"type": "number", "minimum": 0},
            "itemNo": {"type": "string"},
            "contents": {"type": "string"},
            "reference": {"type": "string"},
            "customLabelText1": {"type": "string"},
            "customLabelText2": {"type": "string"},
            "customLabelText3": {"type": "string"},
            "customLabelText4": {"type": "string"},
            "customLabelText5": {"type": "string"},
            "customLabelText6": {"type": "string"},
            "dangerousGoods": {
                "type": "object",
                "properties": {
                    "unCode": {"type": "string"},
                    "hazardCode": {"type": "string"},
                    "packageCode": {"type": "string"},
                    "packageType": {"type": "string"},
                    "description": {"type": "string"},
                    "adrClass": {"type": "string"},
                    "mpCode": {"type": "string"},
                    "ems": {"type": "string"},
                    "note": {"type": "string"},
                    "netWeight": {"type": "number"},
                    "netVolume": {"type": "number"},
                    "trCode": {"type": "string"},
                    "flashPoint": {"type": "string"},
                    "limitedQuantities": {"type": "boolean"},
                    "separation": {"type": "string"}
                }
            },
            "articles": {
                "type": "array",
                "items": {
                    "type": "object",
                    "properties": {
                        "articleNo": {"type": "string"},
                        "count": {"type": "integer"},
                        "name": {"type": "string"},
                        "enot": {"type": "boolean"},
                        "price": {"type": "number", "minimum": 0},
                        "currencyCode": {"type": "string"},
                        "weight": {"type": "number", "minimum": 0},
                        "volume": {"type": "number", "minimum": 0},
                        "loadingMeters": {"type": "number", "minimum": 0},
                        "contents": {"type": "string"},
                        "marking": {"type": "string"},
                        "packageCode": {"type": "string"},
                        "customsStatNo": {"type": "string"},
                        "customsSubStatNo1": {"type": "string"},
                        "customsOtherUnit": {"type": "string"},
                        "customsOtherQuantity": {"type": "number", "minimum": 0},
                        "customsValue": {"type": "number", "minimum": 0},
                        "customsContents": {"type": "string"},
                        "customsNetWeight": {"type": "number", "minimum": 0},
                        "customsSourceCountryCode": {"type": "string"},
                        "dangerousGoods": {
                            "type": "object",
                            "properties": {
                                "unCode": {"type": "string"},
                                "hazardCode": {"type": "string"},
                                "packageCode": {"type": "string"},
                                "packageType": {"type": "string"},
                                "description": {"type": "string"},
                                "adrClass": {"type": "string"},
                                "mpCode": {"type": "string"},
                                "ems": {"type": "string"},
                                "note": {"type": "string"},
                                "netWeight": {"type": "number"},
                                "netVolume": {"type": "number"},
                                "trCode": {"type": "string"},
                                "flashPoint": {"type": "string"},
                                "limitedQuantities": {"type": "boolean"},
                                "separation": {"type": "string"}
                            }
                        }
                    }
                }
            }
        }
    }
}

SERVICE_SCHEMA = {
    "type": "object",
    "required": ["id"],
    "properties": {
        "id": {"type": "string"},
        "subId": {"type": "string"},
        "normalShipment": {"type": "boolean"},
        "returnShipment": {"type": "boolean"},
        "referenceAsBarcode": {"type": "boolean"},
        "nonDeliveryType": {"type": "string"},
        "valueAmount": {"type": "string"},
        "valueCurrencyCode": {"type": "string"},
        "paymentMethodType": {"type": "string"},
        "sortPos": {"type": "string"},
        "destinationLocation": {"type": "string"},
        "notifyCode1": {"type": "string"},
        "notifyCode2": {"type": "string"},
        "notifyCode3": {"type": "string"},
        "bookingId": {"type": "string"},
        "bookingOffice": {"type": "string"},
        "infoCode": {"type": "string"},
        "contractVersion": {"type": "string"},
        "terminal": {"type": "string"},
        "handOverCode": {"type": "string"},
        "externalIdentifier": {"type": "string"},
        "waybillInvoice": {"type": "integer", "minimum": 0},
        "waybillEurCertificate": {"type": "integer", "minimum": 0},
        "waybillExportNotification": {"type": "integer", "minimum": 0},
        "waybillUnits332": {"type": "integer", "minimum": 0},
        "waybillWeight332": {"type": "number", "minimum": 0},
        "waybillUnits334": {"type": "integer", "minimum": 0},
        "waybillWeight334": {"type": "number", "minimum": 0},
        "waybillUnits336": {"type": "integer", "minimum": 0},
        "waybillWeight336": {"type": "number", "minimum": 0},
        "waybillUnits342": {"type": "integer", "minimum": 0},
        "waybillWeight342": {"type": "number", "minimum": 0},
        "waybillCod": {"type": "integer", "minimum": 0},
        "waybillCod342": {"type": "integer", "minimum": 0},
        "waybillHomeDelivery342": {"type": "integer", "minimum": 0},
        "shipperLoadAndCount": {"type": "integer", "minimum": 0},
        "pickupBooking": {"type": "boolean"},
        "pickupDate": {"type": "string"},
        "pickupTimeFrom": {"type": "string"},
        "pickupTimeTo": {"type": "string"},
        "pickupText1": {"type": "string"},
        "pickupMisc": {"type": "string"},
        "addons": {
            "type": "array",
            "items": {
                "type": "object",
                "required": ["id"],
                "properties": {
                    "id": {"type": "string"},
                    "amount": {"type": "number", "minimum": 0},
                    "account": {"type": "string"},
                    "accountType": {"type": "string"},
                    "bank": {"type": "string"},
                    "currencyCode": {"type": "string"},
                    "custNo": {"type": "string"},
                    "custNoIssuerCode": {"type": "string"},
                    "misc": {"type": "string"},
                    "miscType": {"type": "string"},
                    "contact": {"type": "string"},
                    "reference": {"type": "string"},
                    "referenceType": {"type": "string"},
                    "tempMin": {"type": "number"},
                    "tempMax": {"type": "number"},
                    "email": {"type": "string"},
                    "text1": {"type": "string"},
                    "text2": {"type": "string"},
                    "text3": {"type": "string"},
                    "text4": {"type": "string"},
                    "text5": {"type": "string"},
                    "text6": {"type": "string"},
                    "text7": {"type": "string"},
                    "text8": {"type": "string"},
                    "text9": {"type": "string"},
                    "text10": {"type": "string"},
                    "length": {"type": "number", "minimum": 0},
                    "width": {"type": "number", "minimum": 0},
                    "declarant": {"type": "string"},
                    "passengerFlight": {"type": "boolean"},
                    "cargoFlight": {"type": "boolean"},
                    "documentType": {"type": "string"}
                }
            }
        }
    }
}

EXTRAS_SCHEMA = {
    "type": "object",
    "properties": {
        "favorite": {"type": "string"},
        "profileGroup": {"type": "string"},
        "note": {"type": "string"},
        "test": {"type": "boolean"},
        "linkPrintKey": {"type": "string"},
        "orderNo": {"type": "string"},
        "mergeId": {"type": "string"},
        "freeText1": {"type": "string"},
        "freeText2": {"type": "string"},
        "freeText3": {"type": "string"},
        "freeText4": {"type": "string"},
        "senderReference": {"type": "string"},
        "receiverReference": {"type": "string"},
        "goodsDescription": {"type": "string"},
        "bulkId": {"type": "string"},
        "totalEurPallets": {"type": "integer", "minimum": 0},
        "totalHalfPallets": {"type": "integer", "minimum": 0},
        "totalQuarterPallets": {"type": "integer", "minimum": 0},
        "totalWeight": {"type": "number", "minimum": 0},
        "totalVolume": {"type": "number", "minimum": 0},
        "totalLoadingMeters": {"type": "number", "minimum": 0},
        "totalSortCode": {"type": "string"},
        "totalQuantity": {"type": "integer", "minimum": 0},
        "totalPieces": {"type": "integer", "minimum": 0},
        "totalPallets": {"type": "integer", "minimum": 0},
        "waybillFreeText1": {"type": "string"},
        "waybillFreeText2": {"type": "string"},
        "waybillFreeText3": {"type": "string"},
        "waybillFreeText4": {"type": "string"},
        "waybillFreeText5": {"type": "string"},
        "waybillSpecialAgreement": {"type": "string"},
        "waybillDocuments1": {"type": "string"},
        "waybillDocuments2": {"type": "string"},
        "waybillCondition": {"type": "string"},
        "termsCode": {"type": "string"},
        "termsLocation": {"type": "string"},
        "termsLocationIdentifier": {"type": "string"},
        "printSet": {
            "type": "array",
            "items": {
                "type": "string"
            }
        },
        "shipDate": {"type": "string", "format": "date-time"},
        "pickupTime": {"type": "string", "format": "date-time"},
        "deliveryDate": {"type": "string", "format": "date-time"},
        "deliveryTimeEarliest": {"type": "string", "format": "date-time"},
        "deliveryTimeLastest": {"type": "string", "format": "date-time"},
        "shipmentNo": {"type": "string"},
        "ediForward": {"type": "boolean"},
        "tplFormat": {"type": "boolean"},
        "pdfInsert": {"type": "string"},
        "deliveryInstruction": {"type": "string"},
        "customLabelText1": {"type": "string"},
        "customLabelText2": {"type": "string"},
        "customLabelText3": {"type": "string"},
        "customLabelText4": {"type": "string"},
        "customLabelText5": {"type": "string"},
        "customLabelText6": {"type": "string"},
        "options": {
            "type": "array",
            "items": {
                "type": "object",
                "required": ["id"],
                "properties": {
                    "id": {"type": "string"},
                    "from": {"type": "string"},
                    "to": {"type": "string"},
                    "cc": {"type": "string"},
                    "bcc": {"type": "string"},
                    "errorTo": {"type": "string"},
                    "message": {"type": "string"},
                    "languageCode": {"type": "string"},
                    "mailTemplate": {"type": "string"},
                    "sendEmail": {"type": "boolean"}
                }
            }
        }
    }
}

PDF_CONFIG_SCHEMA = {
    "type": "object",
    "required": ["target1Media"],
    "properties": {
        "target1Media": {"type": "string"},
        "target1XOffset": {"type": "number"},
        "target1YOffset": {"type": "number"},
        "target2Media": {"type": "string"},
        "target2XOffset": {"type": "number"},
        "target2YOffset": {"type": "number"},
        "target3Media": {"type": "string"},
        "target3XOffset": {"type": "number"},
        "target3YOffset": {"type": "number"},
        "target4Media": {"type": "string"},
        "target4XOffset": {"type": "number"},
        "target4YOffset": {"type": "number"}
    }
}

REQUEST_SCHEMA = {
    "type": "object",
    "$schema": "http://json-schema.org/draft-04/schema#",
    "required": ["shipment", "pdfConfig"],
    "properties": {
        "shipment": {
            "allOf": [
                {
                    "type": "object",
                    "required": ["sender", "receiver", "service", "parcels"],
                    "properties": {
                        "sender": ADDRESS_SCHEMA,
                        "senderPartners": PARTNER_SCHEMA,
                        "dispatch": ADDRESS_SCHEMA,
                        "receiver": ADDRESS_SCHEMA,
                        "receiverPartners": PARTNER_SCHEMA,
                        "delivery": ADDRESS_SCHEMA,
                        "agent": ADDRESS_SCHEMA,
                        "returnPart": ADDRESS_SCHEMA,
                        "freightPayer": ADDRESS_SCHEMA,
                        "taxPayer": ADDRESS_SCHEMA,
                        "customsPayer": ADDRESS_SCHEMA,
                        "service": SERVICE_SCHEMA,
                        "parcels": PARCELS_SCHEMA,
                        "customsDeclaration": CUSTOMS_DECLARATION_SCHEMA
                    }
                },
                EXTRAS_SCHEMA
            ]

        },
        "pdfConfig": PDF_CONFIG_SCHEMA
    }
}

RESPONSE_SCHEMA = {
    "type": "array",
    "$schema": "http://json-schema.org/draft-04/schema#",
    "items": {
        "type": "object",
        "required": ["href", "id"],
        "properties": {
            "href": {"type": "string"},
            "id": {"type": "string"},
            "status": {"type": "string"},
            "shipmentNo": {"type": "string"},
            "orderNo": {"type": "string"},
            "reference": {"type": "string"},
            "serviceId": {"type": "string"},
            "parcelCount": {"type": "number"},
            "sndName": {"type": "string"},
            "sndZipcode": {"type": "string"},
            "sndCity": {"type": "string"},
            "sndCountry": {"type": "string"},
            "rcvName": {"type": "string"},
            "rcvZipcode": {"type": "string"},
            "rcvCity": {"type": "string"},
            "rcvCountry": {"type": "string"},
            "created": {"type": "string", "format": "date-time"},
            "changed": {"type": "string", "format": "date-time"},
            "shipDate": {"type": "string", "format": "date-time"},
            "returnShipment": {"type": "boolean"},
            "normalShipment": {"type": "boolean"},
            "consolidated": {"type": "boolean"},
            "parcels": {
                "type": "array",
                "items": {
                    "type": "object",
                    "properties": {
                        "parcelNo": {"type": "string"},
                        "reference": {"type": "string"}
                    }
                }
            },
            "pdfs": {
                "type": "array",
                "items": {
                    "type": "object",
                    "required": ["href", "id"],
                    "properties": {
                        "href": {"type": "string"},
                        "id": {"type": "string"},
                        "description": {"type": "string"}
                    }
                }
            },
            "previousPdfs": {
                "type": "array",
                "items": {
                    "type": "object",
                    "required": ["href", "id"],
                    "properties": {
                        "href": {"type": "string"},
                        "id": {"type": "string"},
                        "description": {"type": "string"}
                    }
                }
            }
        }
    }
}

LOCATION_ADDRESS_SCHEMA = {
    "type": "object",
    "properties": {
        "address": {"type": ["string", "null"]},
        "municipality": {"type": ["string", "null"]},
        "postalCode": {"type": ["string", "null"]},
        "postalCodeName": {"type": ["string", "null"]},
        "streetName": {"type": ["string", "null"]},
        "streetNumber": {"type": ["string", "null"]},
    }
}

LOCATION_ADDRESS_NAME_SCHEMA = {
    "type": "object",
    "properties": {
        "en": LOCATION_ADDRESS_SCHEMA,
        "fi": LOCATION_ADDRESS_SCHEMA,
        "sv": LOCATION_ADDRESS_SCHEMA,
    },
}

LOCATION_NAME_SCHEMA = {
    "type": ["object", "null"],
    "properties": {
        "en": {"type": ["string", "null"]},
        "fi": {"type": ["string", "null"]},
        "sv": {"type": ["string", "null"]},
    }
}

LOCATION_OPENING_TIME_SCHEMA = {
    "type": "object",
    "properties": {
        "timeFrom": {"type": "string"},
        "timeFromWithPoint": {"type": "string"},
        "timeTo": {"type": "string"},
        "timeToWithPoint": {"type": "string"},
        "weekday": {"type": "string"},
    }
}

LOCATION_SCHEMA = {
    "type": "object",
    "properties": {
        "id": {"type": "string"},
        "type": {"type": "string"},
        "locationName": LOCATION_NAME_SCHEMA,
        "publicName": LOCATION_NAME_SCHEMA,
        "labelName": LOCATION_NAME_SCHEMA,
        "additionalInfo": LOCATION_NAME_SCHEMA,
        "postalCode": {"type": ["string", "null"]},
        "postalCodeAreas": {"type": ["array", "null"]},
        "address": LOCATION_ADDRESS_NAME_SCHEMA,
        "countryCode": {"type": "string"},
        "location": {
            "type": "object",
            "properties": {
                "lat": {"type": "string"},
                "lon": {"type": "string"},
            },
        },
        "openingTimes": {
            "type": ["array", "null"],
            "items": LOCATION_OPENING_TIME_SCHEMA,
        },
        "wheelChairAccess": {"type": "boolean"},
        "dropOfTimeParcel": {"type": ["string", "null"]},
        "dropOfTimeLetters": {"type": ["string", "null"]},
        "dropOfTimeExpress": {"type": ["string", "null"]},
        "pupCode": {"type": ["string", "null"]},
        "routingServiceCode": {"type": ["string", "null"]},
        "postalOfficeType": {"type": ["string", "null"]},
        "availability": {"type": ["string", "null"]},
        "partnerType": {"type": ["string", "null"]},
        "category": {"type": ["string", "null"]},
        "emptyTime": {"type": ["string", "null"]},
        "letterClass": {"type": ["string", "null"]},
        "capacity": {"type": ["string", "null"]},
    },
}

LOCATIONS_SCHEMA = {
    "type": "array",
    "$schema": "http://json-schema.org/draft-04/schema#",
    "items": LOCATION_SCHEMA,
}



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import attr
import six
from jsonschema import validate

from .constants import ResponseCode
from .exceptions import ApiError
from .objects import (
    Agent, Parcels, PDFConfig, Receiver, Sender, SenderPartners, Service)
from .schemas import REQUEST_SCHEMA

DEFAULT_PDF_CONFIG = {
    "target1Media": "laser-a5",
    "target1YOffset": 0,
    "target1XOffset": 0,
}


@attr.s
class Shipment(object):
    agent = attr.ib(default=Agent())
    orderNo = attr.ib(default=None)
    sender = attr.ib(default=Sender())
    senderPartners = attr.ib(default=SenderPartners())
    receiver = attr.ib(default=Receiver())
    parcels = attr.ib(default=Parcels())
    service = attr.ib(default=Service())
    senderReference = attr.ib(default="")
    freeText1 = attr.ib(default="")
    # TODO: add remaining attributes

    data = attr.ib(default={})
    pdfConfig = attr.ib(default=PDFConfig())

    def build(self):
        """
        Build and validate the data for a Shipment.
        """
        data = {
            "shipment": {
                "sender": self.sender.get_json(),
                "senderPartners": self.senderPartners.get_json(),
                "parcels": self.parcels.get_json(),
                "receiver": self.receiver.get_json(),
                "service": self.service.get_json(),
            }
        }
        pdf_config = self.pdfConfig.get_json()
        data["pdfConfig"] = pdf_config if pdf_config else DEFAULT_PDF_CONFIG
        agent = self.agent.get_json()
        if agent:
            data["shipment"]["agent"] = agent
        if self.orderNo:
            data["shipment"]["orderNo"] = self.orderNo
        if self.senderReference:
            data["shipment"]["senderReference"] = self.senderReference
        if self.freeText1:
            data["shipment"]["freeText1"] = self.freeText1
        # TODO: set remaining attributes, if given
        # Drop top-level key's with empty value
        self.data = dict((key, value) for key, value in six.iteritems(data) if value)
        validate(self.data, REQUEST_SCHEMA)


class ShipmentResponseError(ApiError):
    pass


class ShipmentResponse(object):
    def __init__(self, response):
        try:
            self.response_code = ResponseCode(response.status_code)
        except ValueError:
            raise ValueError("Unknown response status %d" % response.status_code)
        self.raw = response

    def raise_for_status(self):
        error_message = None
        if self.response_code is ResponseCode.MISSING:
            error_message = "Missing required attribute: %s" % self.raw.json()["message"]
        elif self.response_code is ResponseCode.UNAUTHORIZED:
            error_message = "Unauthorized API use: %s" % self.raw.reason
        elif self.response_code is ResponseCode.VALIDATION_ERROR:
            fields = ",".join(str(error["field"]) for error in self.raw.json())
            error_message = "Validation failed on fields: %s" % fields
        elif self.response_code is ResponseCode.SERVER_ERROR:
            try:
                data = self.raw.json()
                if "message" not in data:
                    error_message = "Invalid server response"
                else:
                    error_message = "Internal server error: %s" % data["message"]
            except ValueError as e:
                error_message = str(e)

        if error_message is not None:
            raise ShipmentResponseError(error_message, self.response_code, self.raw)

    def get_pdfs(self, client):
        """
        Extract PDF data from the response or fetch it if missing.

        :param client: Smartship client with valid credentials
        :type client: smartship.Client
        :return: PDF-data for each response fragment
        :rtype: list[list[bytes]]
        :raises: ShipmentResponseError: if the response has the wrong status code for a shipment
        :raises: KeyError: if the the JSON of the response doesn't conform to the schema
        """
        if self.response_code != ResponseCode.CREATED:
            raise ShipmentResponseError(
                "Invalid shipment response status code: %r" % self.response_code, self.response_code, self.raw
            )

        # TODO: Add logging
        results = []
        for fragment in self.raw.json():
            result = []
            shipment_id = fragment["id"]
            for pdf in fragment["pdfs"]:
                # Check if PDF data was already included in the response
                if pdf.get("pdf"):
                    # Use existing data
                    result.append(pdf["pdf"])
                else:
                    # Fetch missing data
                    result.append(client.get_pdf(shipment_id, pdf["id"]))
            results.append(result)
        return results

    @property
    def data(self):
        return self.raw.json()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import requests
from requests.auth import HTTPBasicAuth

from .constants import ResponseCode
from .exceptions import ApiError
from .objects import Agents
from .shipments import ShipmentResponse


class Client(object):
    """
    Used for communicating with the Unifaun Online REST API.
    """
    SMARTSHIP_API_ENDPOINT = "https://api.unifaun.com/rs-extapi/v1"

    def __init__(self, username, secret, api_url=None):
        self._username = username
        self._secret = secret
        if api_url:
            self._api_url = api_url
        else:
            self._api_url = self.SMARTSHIP_API_ENDPOINT

    def send_shipment(self, shipment):
        """
        Build and send a Shipment.

        :param shipment: Shipment instance
        :type shipment: smartship.shipments.Shipment
        :return: ShipmentResponse
        """
        shipment.build()
        # TODO: Add logging
        response = self._post("/shipments", shipment.data)
        shipment_response = ShipmentResponse(response)
        shipment_response.raise_for_status()
        return shipment_response

    def get_pdf(self, shipment_id, pdf_id):
        """
        Get specified PDF data for a shipment

        :param shipment_id: Shipment identifier from a shipment response
        :type shipment_id: str
        :param pdf_id: PDF identifier from a shipment response
        :type pdf_id: str
        :return: bytes
        """
        endpoint = "/shipments/%s/pdfs/%s" % (shipment_id, pdf_id)
        pdf_response = self._get(endpoint)
        if pdf_response.status_code != ResponseCode.OK:
            raise ApiError(
                "Invalid PDF response status code: %r" % pdf_response.status_code,
                pdf_response.status_code,
                pdf_response
            )
        return pdf_response.content

    def get_agents(self, country_code, agent_type, street=None, zipcode=None, agent_id=None):
        """
        Get agents (pickup location) information.

        Example using receiver address to get the closest ones:
            agents = client.get_agents("FI", "ITELLASP", "Iso Roobertinkatu 20-22", "00120")
        Example using the agent id:
            agents = client.get_agents("SE", "POSTNORD", agent_id="586181")

        :param country_code: Two letter country code (from ISO-3166-1 alpha-2).
        :type country_code: str
        :param agent_type: Type of the agent. One of
            (SBTL, SBTLFI, PP, PPFI, POSTNORD, DHLSP, DHLSPCOD, GLS, MHM, MHT, ITELLA, ITELLASP, BRING, BUSSGODS, UPS)
        :type agent_type: str
        :param street: The street name and number of the receiver's address for finding the closest agent.
            NOTE: Can't be used with the agent_id parameter.
        :type street: str
        :param zipcode: The zipcode code of the receiver's address. NOTE: Can't be used with the agent_id parameter.
        :type zipcode: str
        :param agent_id: The ID of the agent used by the carrier. NOTE: Can't be used with zipcode or street parameters.
        :type agent_id: str
        :return: Agents
        """
        if agent_id and (street or zipcode):
            raise ValueError("Agent ID and receiver address parameters cannot be used simultaneously")
        params = {
            "countryCode": country_code,
            "type": agent_type,
        }
        if agent_id:
            params["id"] = agent_id
        else:
            params["street"] = street
            params["zip"] = zipcode

        response = self._get("/addresses/agents", params)
        status_code = response.status_code
        if status_code == ResponseCode.MISSING:
            raise ApiError(response.json()["message"], status_code, response)
        elif status_code != ResponseCode.OK:
            raise ApiError("Error getting agents information", status_code, response)

        return Agents(response.json())

    def _post(self, endpoint, data):
        """
        Send a post request to the Posti SmartShip/Unifaun API.

        :param endpoint: API endpoint for desired API resource (for example `/shipments`)
        :type endpoint: string
        :type data: dict
        :return: response received from Posti SmartShip/Unifaun
        :rtype: HttpResponse
        """
        response = requests.post(
            "%s%s" % (self._api_url, endpoint),
            json=data,
            auth=HTTPBasicAuth(self._username, self._secret)
        )
        # TODO: Add error logging
        return response

    def _get(self, endpoint, params=None):
        """
        Get a resource from the Posti SmartShip/Unifaun API.

        :param endpoint: API endpoint for desired API resource (for example `/shipments/<id>/pdfs`)
        :type endpoint: string
        :type params: dict
        :return: response received from Posti SmartShip/Unifaun
        :rtype: HttpResponse
        """
        response = requests.get(
            "%s%s" % (self._api_url, endpoint),
            params=params,
            auth=HTTPBasicAuth(self._username, self._secret),
        )
        # TODO: Add logging
        return response



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from copy import deepcopy

import requests

from ..objects import (
    Agent, Locations, Parcels, PDFConfig, Receiver, Sender, SenderPartners,
    Service)
from ..shipments import Shipment

CARRIER_CODE = "POSTI"
CARRIER_DESCRIPTION = "Posti Oy, Paketit ja kuljetusyksiköt"

SERVICES = {
    "IT14I": "Posti - Express Business Day parcel (Ulkomaa)",
    "ITKY14I": "Posti - Express Business Day pallet (Ulkomaa)",
    "ITPR": "Posti - Priority Parcel",
    "PO2017": "Posti - EMS",
    "PO2017D": "Posti - EMS DocPack",
    "PO2102": "Posti - Express-paketti",
    "PO2103": "Posti - Postipaketti",
    "PO2104": "Posti - Kotipaketti",
    "PO2108": "Posti - Palautus",
    "PO2114": "Posti - Ruokakuljetus",
    "PO2115": "Posti - Ruokakuljetus automaattiin",
    "PO2144": "Posti - Express-rahti",
    "PO2331": "Posti - Postal Parcel Baltics",
    "PO2338": "Posti - Postal Parcel Baltics Return",
    "PO2461": "Posti - Pikkupaketti",
    "PO2711": "Posti - Parcel Connect",
    "PO2718": "Posti - Parcel Return Connect",
    "PO5001": "Posti - Pikakirje",
    "PO5002": "Posti - Economy-kirje postiennakolla",
    "PO5003": "Posti - Kirjattu kirje",
    "PO5004": "Posti - Saantitodistuskirje",
    "PO5006": "Posti - Postivakuutettu",
    "PO5007": "Posti - Priority-kirje postiennakolla",
}

NATIONAL_SERVICE_KEYS = [
    "ITPR",
    "PO2102",
    "PO2103",
    "PO2104",
    "PO2108",
    "PO2114",
    "PO2115",
    "PO2144",
    "PO2461",
    "PO5001",
    "PO5002",
    "PO5003",
    "PO5004",
    "PO5006",
    "PO5007",
]

NATIONAL_SERVICES = {
    key: description
    for (key, description) in SERVICES.items()
    if key in NATIONAL_SERVICE_KEYS
}

ADDITIONAL_SERVICES = {
    "COD": "Postiennakko",
    "COLD": "Jääkaappilämpö",
    "DLV": "Kotiinkuljetus",
    "DLV00": "Samana päivänä 00",
    "DLV09": "Aamuksi 09",
    "DLV21": "Illaksi 21",
    "DLVCALL": "Soitto ennen jakelua",
    "DLVANOT": "Jakeluajankohdan sopiminen puhelimitse",
    "DLVDEP": "Toimitus terminaaliin",
    "DLVINST": "Käyttökuntoon asennus",
    "DLVNOPOD": "Luovuttaminen ilman vastaanottajan kuittausta",
    "DLVPRIV": "Jakelu yksityishenkilölle",
    "DLVSAT": "Lauantaijakelu",
    "DLVT": "Aikataulutettu jakelu",
    "DNG": "Vaarallisten aineiden kuljetus (VAK) / LQ Kuljetus",
    "FDNGPP": "LQ Prosessilupa",
    "FRAG": "Särkyvä",
    "FRZ": "Pakaste",
    "INSU": "Kuljetusvakuutus",
    "MAXI": "Maksikoko",
    "MPRC": "Monikollilähetys",
    "NOBOX": "Ohjauksen esto ulkoautomaattiin",
    "NOT": "Sähköinen saapumisilmoitus",
    "NOTLTR": "Saapumisilmoitus kirjeenä",
    "OPAY": "Maksaja muu kuin lähettäjä",
    "PERS": "Henkilökohtaisesti luovutettava",
    "PRENOT": "Sähköinen ennakkoilmoitus",
    "PUPDEP": "Nouto terminaalista",
    "PUPOPT": "Vaihtoehtoinen noutopiste",
    "RBLOCK": "Pakettiohjauksen esto",
    "RECYCLE": "Kuljetus kierrätykseen",
    "REG": "Kirjaaminen",
    "REMIPOST": "Noutomuistutus kirjeenä",
    "RETNEXT": "Säilytysajan pidennys",
    "SPTR": "Suri",
    "TECH": "Nosturipalvelu",
    "WARM": "Lämminkuljetus",
}

VALID_ADDITIONAL_SERVICES = {
    'IT14I': ['COD', 'DLV', 'DLVCALL', 'DLVNOPOD', 'FDNGPP', 'FRAG', 'MPRC', 'OPAY', 'SPTR'],
    'ITKY14I': [
        'COD', 'DLV00', 'DLV09', 'DLV21', 'DLVCALL', 'DLVINST', 'DLVNOPOD', 'DNG', 'MPRC',
        'OPAY', 'PERS', 'PRENOT', 'PUPDEP', 'RECYCLE', 'SPTR'],
    'ITPR': ['OPAY'],
    'PO2017': ['OPAY'],
    'PO2017D': ['OPAY'],
    'PO2102': [
        'COD', 'DLV00', 'DLV09', 'DLVCALL', 'DLVINST', 'DLVNOPOD', 'DLVSAT', 'DNG',
        'FDNGPP', 'FRAG', 'NOT', 'OPAY', 'PERS', 'PRENOT', 'RBLOCK', 'RECYCLE', 'SPTR'
    ],
    'PO2103': [
        'COD', 'FDNGPP', 'FRAG', 'NOBOX', 'NOT', 'NOTLTR', 'OPAY', 'PERS', 'PUPOPT',
        'RBLOCK', 'REG', 'REMIPOST', 'RETNEXT', 'SPTR'],
    'PO2104': ['COD', 'DLVNOPOD', 'DLVANOT', 'DNG', 'FDNGPP', 'FRAG', 'OPAY', 'PERS', 'SPTR'],
    'PO2108': ['FDNGPP', 'FRAG', 'OPAY', 'SPTR'],
    'PO2114': ['COLD', 'OPAY', 'FDNGPP', 'FRZ'],
    'PO2115': ['COLD', 'OPAY', 'FDNGPP', 'FRZ'],
    'PO2144': [
        'COD', 'DLV00', 'DLV09', 'DLV21', 'DLVANOT', 'DLVCALL', 'DLVINST', 'DLVNOPOD', 'DNG',
        'OPAY', 'PERS', 'PRENOT', 'RECYCLE', 'WARM'
    ],
    'PO2331': ['FDNGPP', 'OPAY', 'SPTR', 'FRAG', 'NOT'],
    'PO2338': ['FDNGPP', 'OPAY', 'SPTR', 'FRAG'],
    'PO2461': ['OPAY'],
    'PO2711': ['COD', 'DLV', 'OPAY', 'PUPOPT', 'SPTR'],
    'PO2718': ['OPAY', 'SPTR'],
    'PO5001': ['DLVSAT', 'OPAY'],
    'PO5002': ['COD', 'NOT', 'OPAY', 'REMIPOST'],
    'PO5003': ['NOT', 'OPAY', 'PERS', 'REMIPOST'],
    'PO5004': ['NOT', 'OPAY', 'PERS', 'REMIPOST'],
    'PO5006': ['COD', 'INSU', 'NOT', 'OPAY', 'PERS', 'REMIPOST'],
    'PO5007': ['COD', 'NOT', 'OPAY', 'REMIPOST'],
}

LOCATION_SERVICE_API_ENDPOINT = "https://locationservice.posti.com/location"


class MobileReceiver(Receiver):
    schema = deepcopy(Receiver.schema)
    schema["oneOf"][0] = {"required": ["name", "city", "country", "mobile"]}


class WeightedParcels(Parcels):
    schema = deepcopy(Parcels.schema)
    schema["items"]["required"].append("weight")


def create_shipment(
        custno, service_id, receiver, sender, parcels,
        agent=None, order_no=None, sender_reference=None, pdf_config=None, addons=None, free_text=None,):
    """
    Create a shipment using the Posti carrier.

    Example simplest case usage:
        receiver = {
            "name": "Anders Innovations",
            "city": "Helsinki",
            "country": "FI",
            "address1": "Iso Roobertinkatu 20-22",
            "zipcode": "00120"
        }
        sender = {
            "quickId": "1",
        }
        shipment = create_shipment("12345", "PO2102", receiver, sender, [{"copies": 1}])

    :param custno: Posti customer number.
    :type custno: str
    :param service_id: Service to use, see `SERVICES` constant or API documentation.
    :type service_id: str
    :param receiver: Receiver of the shipment. Must either have a 'quickId' or enough address information.
        See example receiver data above. Required receiver information can depend on service chosen. If necessary,
        check service lists in Unifaun Online and API docs at https://smartship.unifaun.com/rs-docs/##creating_shipments
    :type receiver: dict
    :param parcels: Parcels to send in this shipment. This needs to be a list of items, with at minimum the amount
        of items in the parcel. For example, a shipment with one item:
            [
                {
                    "copies": 1,
                },
            ]
        See full specification of in the Smartship API documentation at
        https://smartship.unifaun.com/rs-docs/##creating_shipments
    :type parcels: list
    :param sender: Sender information. Must either have a 'quickId' or enough address information.
    :type sender: dict
    :param agent: Pickup agent (optional)
    :type agent: dict
    :param order_no: Order number (optional)
    :type order_no: str
    :param sender_reference: Sender reference (optional)
    :type sender_reference: str
    :param pdf_config: PDF config (optional)
    :type pdf_config: dict
    :param addons: Service addons. Available addons depend on chosen service (optional). See Unfifaun Online
        service lists. Defined as a list of dictionaries containing addon 'id' and additional information, for example:
            [
                {
                    "id": "DNG",
                    "declarant": "Firma Oy",
                },
                {
                    "id": "SPTR",
                },
            ]
    :type addons: list
    :param free_text: free text (optional)
    :type free_text: str
    :return: Shipment instance
    :rtype: smartship.shipments.Shipment
    """
    _validate_create_shipment(service_id)
    receiver_class = _infer_receiver_class(service_id, agent)
    parcels_class = _infer_parcels_class(service_id)

    kwargs = {
        "sender": Sender(sender),
        "senderPartners": SenderPartners([{"id": CARRIER_CODE, "custNo": custno}]),
        "receiver": receiver_class(receiver),
        "parcels": parcels_class(parcels),
        "service": _build_service(addons, service_id),
    }
    if agent:
        kwargs["agent"] = Agent(agent)
    if order_no:
        kwargs["orderNo"] = order_no
    if sender_reference:
        kwargs["senderReference"] = sender_reference
    if pdf_config:
        kwargs["pdfConfig"] = PDFConfig(pdf_config)
    if free_text:
        kwargs["freeText1"] = free_text
    return Shipment(**kwargs)


def _build_service(addons, service_id):
    service = Service({"id": service_id})
    if addons:
        service["addons"] = addons
    return service


def _validate_create_shipment(service_id):
    if service_id not in SERVICES:
        raise ValueError("Invalid 'service_id'.")


def _infer_receiver_class(service_id, agent):
    if service_id == "PO2104":
        return MobileReceiver
    if service_id == "PO2103" and agent:
        return MobileReceiver
    return Receiver


def _infer_parcels_class(service_id):
    if service_id == "PO5041":
        return WeightedParcels
    return Parcels


def get_locations(
        country_code=None, top=None,
        types=None, lattitude=None, longitude=None, distance=None, bounding_box=None,
        zipcode=None, location_zipcode=None, strict_zip_code=None, city=None, municipality=None,
        pup_code=None, partner_type=None):
    """
    :param country_code: Limits results to given county. Default value is FI (Finland)
    :type country_code: str
    :param top: Amount of locations to return. API returns this amount of closest locations
    :type top: int
    :param types: supported location types are "POSTOFFICE", "LETTERBOX", "SMARTPOST", "PICKUPPOINT",
        "BUSINESSSERVICE", "POBOX", "LOCKER"
    :type types: list
    :param lattitude: Latitude of center location that search is done for. Top or distance parameter is required.
    :type lattitude: str
    :param : Longitude of center location that search is done for. Top or distance parameter is required.
    :type longitude: str
    :param distance: Filter which allows user to limit distance of returned locations. Parameter is used together with
        lattitude/longitude.
    :type distance: str
    :param bounding_box: Limits results to geographical bounding box.
    :type bounding_box: list
    :param zipcode: Filter results based on zipcode
    :type zipcode: str
    :param location_zipcode: Find the closest pickup points using zipcode
    :type location_zipcode: str
    :param strict_zip_code: This is used together with zip_code parameter. If this is set True only results that
        exactly match to the location zipcode are shown. If strict_zip_code is False, postal code area list is used to
        match location to queried zipcode. Default value is False.
    :type strict_zip_code: bool
    :param city: Filter results based on city. Matches to any language version of the city.
    :type city: str
    :param municipality: Filter results based on municipality. Matches to any language version of the municipality.
    :type municipality: str
    :param pup_code: Filter results based on PupCode
    :type pup_code: str
    :param partner_type: Filter results based on partner type. Allowed types: "POSTI", "AIBE", "BOXNET",
        "TOPO_CENTRAS".
    :type partner_type: str
    """
    params = {
        "countryCode": country_code,
        "top": top,
        "types": types,
        "lat": lattitude,
        "lng": longitude,
        "distance": distance,
        "zipCode": zipcode,
        "locationZipCode": location_zipcode,
        "strictZipCode": "true" if strict_zip_code else None,
        "city": city,
        "municipality": municipality,
        "pupCode": pup_code,
        "partnerType": partner_type,
    }

    if bounding_box is not None:
        params["topLeftLat"] = bounding_box[0]
        params["topLeftLng"] = bounding_box[1]
        params["bottomRightLat"] = bounding_box[2]
        params["bottomRightLng"] = bounding_box[3]

    for key, value in list(params.items()):
        if value is None:
            params.pop(key)

    response = requests.get(LOCATION_SERVICE_API_ENDPOINT, params=params)
    response.raise_for_status()
    return Locations(response.json()["locations"])


def get_additional_services(service_id):
    if service_id not in VALID_ADDITIONAL_SERVICES:
        return {}
    result = {}
    for service_code in VALID_ADDITIONAL_SERVICES[service_id]:
        result[service_code] = ADDITIONAL_SERVICES[service_code]
    return result



# -*- coding: utf-8 -*-
from enum import IntEnum


class ResponseCode(IntEnum):
    OK = 200
    CREATED = 201
    MISSING = 400
    UNAUTHORIZED = 401
    VALIDATION_ERROR = 422
    SERVER_ERROR = 500



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from jsonschema import validate

from .schemas import (
    ADDRESS_SCHEMA, AGENTS_SCHEMA, CUSTOMS_DECLARATION_SCHEMA, EXTRAS_SCHEMA,
    LOCATION_SCHEMA, LOCATIONS_SCHEMA, PARCELS_SCHEMA, PARTNER_SCHEMA,
    PDF_CONFIG_SCHEMA, SERVICE_SCHEMA)


class JSONObject(object):
    schema = {}  # Defined in subclass

    def __init__(self, data=None):
        self._data = data
        if data:
            self.validate()

    def __setitem__(self, key, value):
        self._data[key] = value
        self.validate()

    def __getitem__(self, item):
        return self._data[item]

    def validate(self):
        validate(self._data, self.schema)

    @classmethod
    def from_shipment(cls, shipment):
        data = {}
        return cls(data).get_json()

    def get_json(self):
        return self._data

    def __repr__(self):
        return "%s(%r)" % (self.__class__.__name__, self.get_json())


class Agent(JSONObject):
    schema = ADDRESS_SCHEMA


class Agents(JSONObject):
    schema = AGENTS_SCHEMA


class CustomsDeclaration(JSONObject):
    schema = CUSTOMS_DECLARATION_SCHEMA


class CustomsPayer(JSONObject):
    schema = ADDRESS_SCHEMA


class Extras(JSONObject):
    schema = EXTRAS_SCHEMA


class FreightPayer(JSONObject):
    schema = ADDRESS_SCHEMA


class Parcels(JSONObject):
    schema = PARCELS_SCHEMA


class Delivery(JSONObject):
    schema = ADDRESS_SCHEMA


class Dispatch(JSONObject):
    schema = ADDRESS_SCHEMA


class Receiver(JSONObject):
    schema = ADDRESS_SCHEMA


class ReceiverPartners(JSONObject):
    schema = PARTNER_SCHEMA


class ReturnPart(JSONObject):
    schema = ADDRESS_SCHEMA


class Sender(JSONObject):
    schema = ADDRESS_SCHEMA


class SenderPartners(JSONObject):
    schema = PARTNER_SCHEMA


class Service(JSONObject):
    schema = SERVICE_SCHEMA


class TaxPayer(JSONObject):
    schema = ADDRESS_SCHEMA


class PDFConfig(JSONObject):
    schema = PDF_CONFIG_SCHEMA


class Location(JSONObject):
    schema = LOCATION_SCHEMA


class Locations(JSONObject):
    schema = LOCATIONS_SCHEMA
